@@ -1,17 +1,17 @@
 /* tensor/test_source.c
- * 
+ *
  * Copyright (C) 2002, 2003, 2004, 2007 Jordi Burguet-Castell
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to:
  *   Free Software Foundation, Inc.
@@ -79,10 +79,10 @@ FUNCTION(test, func) (void)
               }
           }
       }
-    
+
     gsl_test (status, NAME (tensor) "_set writes into array");
   }
-  
+
 
 
 
@@ -173,7 +173,7 @@ FUNCTION(test, func) (void)
     /* Test minimum */
     {
       BASE min = FUNCTION(tensor, min) (t);
-      
+
       gsl_test (min != exp_min,
                 NAME(tensor) "_min returns correct minimum value");
     }
@@ -255,7 +255,7 @@ FUNCTION(test, func) (void)
 
       if (imin[2] != exp_kmin)
         status = 1;
-      
+
       gsl_test (status,
                 NAME(tensor) "_minmax_index returns correct indices");
     }
@@ -283,7 +283,7 @@ FUNCTION(test, func) (void)
               }
           }
       }
-    
+
     /* Addition */
     FUNCTION(tensor, memcpy) (t, a);
     FUNCTION(tensor, add) (t, b);
@@ -307,14 +307,14 @@ FUNCTION(test, func) (void)
                 }
             }
         }
-      
+
       gsl_test(status, NAME(tensor) "_add tensor addition");
     }
 
     /* Subtraction */
     FUNCTION(tensor, memcpy) (t, a);
     FUNCTION(tensor, sub) (t, b);
-    
+
     {
       status = 0;
 
@@ -334,7 +334,7 @@ FUNCTION(test, func) (void)
                 }
             }
         }
-      
+
       gsl_test(status, NAME(tensor) "_sub tensor subtraction");
     }
 
@@ -361,7 +361,7 @@ FUNCTION(test, func) (void)
                 }
             }
         }
-      
+
       gsl_test(status,
                NAME(tensor) "_mul_elements elements multiplication");
     }
@@ -389,7 +389,7 @@ FUNCTION(test, func) (void)
                 }
             }
         }
-      
+
       gsl_test(status, NAME(tensor) "_div_elements elements division");
     }
 
@@ -421,7 +421,7 @@ FUNCTION(test, func) (void)
                     if (r != z)
                       status = 1;
                   }
-      
+
       gsl_test(status, NAME(tensor) "_product tensorial product");
 
       FUNCTION(tensor, free) (c);
@@ -429,17 +429,47 @@ FUNCTION(test, func) (void)
 
     /* Index contraction */
     {
-      TYPE(tensor) * tt = FUNCTION(tensor, contract) (a, 0, 1);
+      TYPE(tensor) * t01 = FUNCTION(tensor, contract) (a, 0, 1);
+      TYPE(tensor) * t02 = FUNCTION(tensor, contract) (a, 0, 2);
+      TYPE(tensor) * t12 = FUNCTION(tensor, contract) (a, 1, 2);
 
-      gsl_test(tt->data == 0,
+      gsl_test(t01->data == 0 || t02->data == 0 || t12->data == 0,
                NAME(tensor) "_contract returns valid pointer");
-      gsl_test(tt->rank != RANK-2,
+      gsl_test(t01->rank != RANK-2 || t02->rank != RANK-2 ||
+               t12->rank != RANK-2,
                NAME(tensor) "_contract returns valid rank");
-      gsl_test(tt->dimension != DIMENSION,
+      gsl_test(t01->dimension != DIMENSION || t02->dimension != DIMENSION ||
+               t12->dimension != DIMENSION,
                NAME (tensor) "_contract returns valid dimension");
 
+      status = 0;
 
-      FUNCTION(tensor, free) (tt);
+      for (i = 0; i < DIMENSION; i++)
+        {
+          BASE sum01 = 0;
+          BASE sum02 = 0;
+          BASE sum12 = 0;
+          for (j = 0; j < DIMENSION; j++)
+            {
+              indices[0] = j;  indices[1] = j;  indices[2] = i;
+              sum01 += FUNCTION(tensor, get) (a, indices);
+
+              indices[0] = j;  indices[1] = i;  indices[2] = j;
+              sum02 += FUNCTION(tensor, get) (a, indices);
+
+              indices[0] = i;  indices[1] = j;  indices[2] = j;
+              sum12 += FUNCTION(tensor, get) (a, indices);
+            }
+          if (t01->data[i] != sum01 || t02->data[i] != sum02 ||
+              t12->data[i] != sum12)
+            status = 1;
+        }
+
+      gsl_test(status, NAME(tensor) "_contract indices contraction");
+
+      FUNCTION(tensor, free) (t01);
+      FUNCTION(tensor, free) (t02);
+      FUNCTION(tensor, free) (t12);
     }
 
     /* Swap indices */
@@ -455,20 +485,20 @@ FUNCTION(test, func) (void)
             {
               indices[0] = i;  indices[1] = j;  indices[2] = k;
               BASE x     = FUNCTION(tensor, get) (a,     indices);
-              
+
               indices[0] = j;  indices[1] = i;  indices[2] = k;
               BASE x_102 = FUNCTION(tensor, get) (a_102, indices);
-              
+
               indices[0] = k;  indices[1] = j;  indices[2] = i;
               BASE x_210 = FUNCTION(tensor, get) (a_210, indices);
-              
+
               indices[0] = i;  indices[1] = k;  indices[2] = j;
               BASE x_021 = FUNCTION(tensor, get) (a_021, indices);
-              
+
               if (x != x_102 || x != x_210 || x != x_021)
                 status = 1;
             }
-      
+
       gsl_test(status, NAME(tensor) "_swap_indices swaps indices");
 
       FUNCTION(tensor, free) (a_102);
@@ -508,7 +538,7 @@ FUNCTION(test, text) (void)
             indices[0] = i;  indices[1] = j;  indices[2] = k;
             FUNCTION(tensor, set) (t, indices, (BASE) counter);
           }
-    
+
     FUNCTION(tensor, fprintf) (f, t, OUT_FORMAT);
     fclose(f);
   }
@@ -519,7 +549,7 @@ FUNCTION(test, text) (void)
     status = 0;
 
     FUNCTION(tensor, fscanf) (f, tt);
-    
+
     counter = 0;
     for (i = 0; i < DIMENSION; i++)
       {
@@ -571,7 +601,7 @@ FUNCTION(test, binary) (void)
               }
           }
       }
-    
+
     FUNCTION(tensor, fwrite) (f, t);
     fclose (f);
   }
@@ -583,7 +613,7 @@ FUNCTION(test, binary) (void)
     status = 0;
 
     FUNCTION(tensor, fread) (f, tt);
-    
+
     counter = 0;
     for (i = 0; i < DIMENSION; i++)
       {
@@ -604,7 +634,7 @@ FUNCTION(test, binary) (void)
     fclose(f);
     FUNCTION(tensor, free) (tt);
   }
-  
+
   FUNCTION(tensor, free) (t);
 }
 
@@ -630,7 +660,7 @@ FUNCTION(test, trap) (void)
 
   gsl_test (!status,
             NAME (tensor) "_set traps 1st index above upper bound");
-  
+
 
   status = 0;
   indices[0] = 0;  indices[1] = DIMENSION+1;  indices[2] = 0;
@@ -645,7 +675,7 @@ FUNCTION(test, trap) (void)
 
   gsl_test (!status,
             NAME (tensor) "_set traps 3rd index above upper bound");
-  
+
 
   /* Overflow at upper bound */
 
@@ -679,7 +709,7 @@ FUNCTION(test, trap) (void)
 
   gsl_test (!status,
             NAME (tensor) "_get traps 1st index above upper bound");
-  
+
 
   status = 0;
   indices[0] = 0;  indices[1] = DIMENSION+1;  indices[2] = 0;
@@ -694,7 +724,7 @@ FUNCTION(test, trap) (void)
 
   gsl_test (!status,
             NAME (tensor) "_get traps 3rd index above upper bound");
-  
+
 
   /* Overflow at upper bound */
 
